<?php
// Content Management 0.4 module for PHProjekt, version 3.3.
// Copyright 2002 Mario A. Valdez-Ramirez (mario@mariovaldez.org)
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.



/*
In this file are defined the main OPERATIONS, OBJECTS categories and USER ROLES used
to grant permissions. Because in this role-based schema roles are not combined, we
also define a weight to decide what role's permissions will be considered for a 
given set of conditions (operation, object, user).
The cm_security array is a constraint table made from the defined constants for
operations, objects and roles.
Note that some combinations are invalid (for example, grant permission to delete 
the root page) and will produce an error when executed.

There are two copies of the constraint table; the first one will enable the following
by default:
Only superuser can create first-level dirs.
Superuser cannot post comments anywhere.
Admins can delete and change content only of pages they can view (but they cannot create pages).
Normal users can edit and delete pages they own, no matter if they belong to the viewer group, 
   and they can create pages under their directories.

The second copy of the constraint table is commented out by default. It's the 
representation of the permissions for previous versions of the CM module. If you
uncomment that table you will get:
Admins can create, delete and change content everywhere.
Admins can post only if they belong to the page's viewer group.
Normal users can edit and delete pages they own, no matter if they belong to the viewer group.
This is just in case you want to keep the old behavior.

*/


// Operation
define (CM_SEC_CREATE, "Create");             // Create page
define (CM_SEC_DELETE, "Delete");             // Delete page
define (CM_SEC_CHANGE, "Edit");               // Edit page
define (CM_SEC_DISPLA, "View");               // View page
define (CM_SEC_COMMEN, "Comment");            // Comment on page
// Object
define (CM_SEC_ROOT, "Root");                 // Root directory
define (CM_SEC_SDIR, "SubDir");               // Subdirectory
// User status
define (CM_SEC_SUPERUSER, "SuperUser");       // User is the SuperUser (global role).
define (CM_SEC_ADMINUSER, "AdminUser");       // User is Administrator of any group (global role).
define (CM_SEC_ADMINGROU, "AdminGroup");      // User is Administrator and member of the viewer group (object dependent).
define (CM_SEC_USERGROUP, "UserGroup");       // User is member of the viewer group (object dependent).
define (CM_SEC_USEROWNER, "Owner");           // User is owner of the page (object dependent).
define (CM_SEC_SUPERUSER_W, 100);             // Priority of the user being the SuperUser
define (CM_SEC_ADMINUSER_W, 20);              // Priority of the user being Administrator of any group.
define (CM_SEC_ADMINGROU_W, 60);              // Priority of the user being Administrator and member of the viewer group.
define (CM_SEC_USERGROUP_W, 40);              // Priority of the user being member of the viewer group.
define (CM_SEC_USEROWNER_W, 80);              // Priority of the user being owner of the page.

$cm_security = array();
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;

/*
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_ADMINGROU] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_ROOT][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_ROOT][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_SUPERUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_SUPERUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_ADMINUSER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_ADMINUSER] = false;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_ADMINGROU] = true;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_USERGROUP] = false;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_USERGROUP] = true;
$cm_security[CM_SEC_CREATE][CM_SEC_SDIR][CM_SEC_USEROWNER] = false;
$cm_security[CM_SEC_DELETE][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_CHANGE][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_DISPLA][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
$cm_security[CM_SEC_COMMEN][CM_SEC_SDIR][CM_SEC_USEROWNER] = true;
*/


// fcm_check_constraints
// Lookups the contraints table looking for a given combination of
// conditions. Since we don't know a priori the role of the user
// this function will check what is the role of the user.
function fcm_check_constraints ($cm_operation, $cm_username, $cm_path) {
  global $cm_security, $cm_superuser;
  $cm_location = "";
  $cm_user_status = "";
  $cm_user_status_w = 0;
  if (!$cm_operation)
    $cm_operation = CM_SEC_DISPLA;
  if (!$cm_path) {
    $cm_path = "/";
  }
  if ($cm_path == "/") {
    $cm_location = CM_SEC_ROOT;
  }
  else {
    $cm_location = CM_SEC_SDIR;
  }
  $cm_userisadmin = fcm_user_is_admin ($cm_username);
  $cm_userisingroup = fcm_user_is_ingroup2 ($cm_username, $cm_path);
  if (($cm_username == $cm_superuser) && ($cm_user_status_w < CM_SEC_SUPERUSER_W)) {
    $cm_user_status = CM_SEC_SUPERUSER;
    $cm_user_status_w = CM_SEC_SUPERUSER_W;
  }
  if (($cm_userisadmin) && ($cm_user_status_w < CM_SEC_ADMINUSER_W)) {
    $cm_user_status = CM_SEC_ADMINUSER;
    $cm_user_status_w = CM_SEC_ADMINUSER_W;
  }
  if (($cm_userisingroup) && ($cm_user_status_w < CM_SEC_USERGROUP_W)) {
    $cm_user_status = CM_SEC_USERGROUP;
    $cm_user_status_w = CM_SEC_USERGROUP_W;
  }
  if (($cm_userisingroup && $cm_userisadmin) && ($cm_user_status_w < CM_SEC_ADMINGROU_W)) {
    $cm_user_status = CM_SEC_ADMINGROU;
    $cm_user_status_w = CM_SEC_ADMINGROU_W;
  }
  if ((fcm_user_is_owner ($cm_username, $cm_path)) && ($cm_user_status_w < CM_SEC_USEROWNER_W)) {
    $cm_user_status = CM_SEC_USEROWNER;
    $cm_user_status_w = CM_SEC_USEROWNER_W;
  }
  return ($cm_security[$cm_operation][$cm_location][$cm_user_status]);
};


// fcm_check_validops
// Returns an array with the permissions for the basic operations for
// a given user and location.
function fcm_check_validops ($cm_username, $cm_path) {
  $cm_validops = array ();
  $cm_validops[CM_SEC_CREATE] = fcm_check_constraints (CM_SEC_CREATE, $cm_username, $cm_path);
  $cm_validops[CM_SEC_DELETE] = fcm_check_constraints (CM_SEC_DELETE, $cm_username, $cm_path);
  $cm_validops[CM_SEC_CHANGE] = fcm_check_constraints (CM_SEC_CHANGE, $cm_username, $cm_path);
  $cm_validops[CM_SEC_DISPLA] = fcm_check_constraints (CM_SEC_DISPLA, $cm_username, $cm_path);
  $cm_validops[CM_SEC_COMMEN] = fcm_check_constraints (CM_SEC_COMMEN, $cm_username, $cm_path);
  return ($cm_validops);
};


// fcm_check_anyglobal
// Returns true or false, checking if the user has permissions for a given operation
// using only its global roles (not location dependent) in a generic location.
function fcm_check_anyglobal ($cm_operation, $cm_username, $cm_generic_loc) {
  global $cm_security, $cm_superuser;
  if (($cm_username == $cm_superuser) && ($cm_security[$cm_operation][$cm_generic_loc][CM_SEC_SUPERUSER])) {
    return (true);
  }
  if ((fcm_user_is_admin ($cm_username)) && ($cm_security[$cm_operation][$cm_generic_loc][CM_SEC_ADMINUSER])) {
    return (true);
  }
  return (false);
};


// fcm_user_is_admin
// Check if the user is Administrator.
function fcm_user_is_admin ($cm_username) {
  $result2 = db_query("select kurz, acc from users where kurz = '$cm_username' and acc like '%a%'") or db_die();
  $row2 = db_fetch_row($result2);
  return ($row2[0] <> '');
};


// fcm_user_is_ingroup2
// Check if user is in the group of given a page.
function fcm_user_is_ingroup2 ($cm_username, $cm_path) {
  $result3 = db_query("select cmdb_directory, cmdb_viewer_group, cmdb_type from content3 where cmdb_directory like '$cm_path' and cmdb_type like '%p%'") or db_die();
  $row3 = db_fetch_row($result3);
  $result2 = db_query("select ID, kurz from users where kurz = '$cm_username'") or db_die();
  $row2 = db_fetch_row($result2);
  $result1 = db_query("select user_ID, gruppen.kurz from grup_user, gruppen where
                                        grup_user.user_ID = '$row2[0]' and
                                        grup_user.grup_ID = gruppen.ID and
                                        gruppen.kurz = '$row3[1]'") or db_die();
  $row1 = db_fetch_row($result1);
  return ($row1[0] <> '');
};


// fcm_user_is_owner
// Check if the user is the owner of the page.
function fcm_user_is_owner ($cm_username, $cm_path) {
  $result2 = db_query("select cmdb_directory, cmdb_owner_user, cmdb_type from content3 where cmdb_directory like '$cm_path' and cmdb_owner_user = '$cm_username' and cmdb_type like '%p%'") or db_die();
  $row2 = db_fetch_row($result2);
  return ($row2[0] <> '');
};


?>